import FreeTAKServer.controllers.DatabaseControllers.table_controllers
from FreeTAKServer.controllers.DatabaseControllers import DataPackageTableController, system_user_table_controller
from FreeTAKServer.controllers.DatabaseControllers import VideoStreamTableController
from FreeTAKServer.controllers.DatabaseControllers import EventTableController
from FreeTAKServer.controllers.DatabaseControllers import ActiveEmergencysController
from sqlalchemy.orm import sessionmaker
from sqlalchemy import create_engine
from sqlalchemy.orm import make_transient
from sqlalchemy.exc import IntegrityError
from FreeTAKServer.model.SQLAlchemy.Root import Base
from FreeTAKServer.controllers.configuration.DatabaseConfiguration import DatabaseConfiguration
from FreeTAKServer.controllers.DatabaseControllers import APIUsersController
from FreeTAKServer.controllers.DatabaseControllers import ExCheckController
from FreeTAKServer.controllers.DatabaseControllers import ExCheckChecklistController
from FreeTAKServer.controllers.DatabaseControllers.table_controllers import ActiveFederationsController, UserTableController, FederationsController
import FreeTAKServer.model.SQLAlchemy.CoTTables.Archive
import FreeTAKServer.model.SQLAlchemy.CoTTables._Group
import FreeTAKServer.model.SQLAlchemy.CoTTables.Chat
import FreeTAKServer.model.SQLAlchemy.CoTTables.Color
import FreeTAKServer.model.SQLAlchemy.CoTTables.Contact
import FreeTAKServer.model.SQLAlchemy.CoTTables.Dest
import FreeTAKServer.model.SQLAlchemy.CoTTables.Emergency
import FreeTAKServer.model.SQLAlchemy.CoTTables.Link
import FreeTAKServer.model.SQLAlchemy.CoTTables.Marti
import FreeTAKServer.model.SQLAlchemy.CoTTables.Precisionlocation
import FreeTAKServer.model.SQLAlchemy.CoTTables.Remarks
import FreeTAKServer.model.SQLAlchemy.CoTTables.Serverdestination
import FreeTAKServer.model.SQLAlchemy.CoTTables.Status
import FreeTAKServer.model.SQLAlchemy.CoTTables.Summary
import FreeTAKServer.model.SQLAlchemy.CoTTables.Takv
import FreeTAKServer.model.SQLAlchemy.CoTTables.Track
import FreeTAKServer.model.SQLAlchemy.CoTTables.Uid
import FreeTAKServer.model.SQLAlchemy.CoTTables.Usericon
import FreeTAKServer.model.SQLAlchemy.DataPackage
import FreeTAKServer.model.SQLAlchemy.VideoStream
from FreeTAKServer.model.SQLAlchemy.User import User
import FreeTAKServer.model.SQLAlchemy.APIUsers
import FreeTAKServer.model.SQLAlchemy.system_user
import FreeTAKServer.model.SQLAlchemy.ActiveEmergencys
import FreeTAKServer.model.SQLAlchemy.ExCheckKeywords
import FreeTAKServer.model.SQLAlchemy.ExCheckData
import FreeTAKServer.model.SQLAlchemy.ExCheck
import FreeTAKServer.model.SQLAlchemy.ExCheckChecklist

class DatabaseController:
    """
    this controller should only be instantiated once for
    each database connection to be established. this
    function is the entry point for any classes attempting
    to access the database
    """

    def __init__(self):
        self.engine = self.create_engine()
        self.SessionMaker = self.create_Sessionmaker()
        self.session = self.create_Session()
        self.DataPackageController = DataPackageTableController.DataPackageTableController()
        self.UserTableController = UserTableController()
        self.SystemUserTableController = system_user_table_controller.SystemUserTableController()
        self.VideoStreamTableController = VideoStreamTableController.VideoStreamTableController()
        self.EventTableController = EventTableController.EventTableController()
        self.APIUserController = APIUsersController.APIUsersController()
        self.ActiveEmergencysController = ActiveEmergencysController.ActiveEmergencysController()
        self.ExCheckController = ExCheckController.ExCheckController()
        self.ExCheckChecklistController = ExCheckChecklistController.ExCheckChecklistController()
        self.ActiveFederationController = ActiveFederationsController()
        self.FederationController = FederationsController()

    def create_engine(self):
        """
        this function creates the engine and applies all the metadata
        of classes which inherit the base class and applies it to the database
        to create tables.
        :arg
        """
        engine = create_engine(DatabaseConfiguration().DataBaseConnectionString, echo=False)
        if engine.dialect.has_table(engine, 'SystemUser') == False:
            Base.metadata.create_all(engine)
            tempsession = sessionmaker(bind=engine)()
            tempsession.add(FreeTAKServer.model.SQLAlchemy.system_user.SystemUser(uid="1", name="admin", password="password", token="token"))
            tempsession.commit()
            tempsession.close()
            return engine
        else:
            Base.metadata.create_all(engine)
            return engine
    def create_Sessionmaker(self):
        SessionMaker = sessionmaker(bind=self.engine)
        return SessionMaker

    def create_Session(self):
        """
        this function creates the session with the server
        via the engine
        :param engine:
        :return:
        """
        session = self.SessionMaker()
        return session

    def create_datapackage(self, **args):
        return self._create(self.DataPackageController, **args)

    def remove_datapackage(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which datapackages are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.DataPackageController, query=query)

    def query_datapackage(self, query="1 == 1", column=['*']):
        return self._query(controller=self.DataPackageController, query=query, columns=column)

    def update_datapackage(self, column_value=None, query="1 == 1"):
        return self._update(controller=self.DataPackageController, query=query, column_value=column_value)

    def _create(self, controller, **args):
        controller.create(session=self.session, **args)
        return 1

    def _remove(self, controller, query):
        return controller.delete(session=self.session, query=query)

    def _query(self, controller, query, columns):
        return controller.query(session=self.session, query=query, columns = columns)

    def _update(self, controller, column_value, query):
        return controller.update(session = self.session, column_value=column_value, query=query)

    def create_user(self, **args):
        try:
            return self._create(controller=self.UserTableController, **args)
        except IntegrityError as e:
            if str(e) == """(sqlite3.IntegrityError) UNIQUE constraint failed: User.uid
[SQL: INSERT INTO "User" (uid, callsign, "CN", "IP", "CoT_id") VALUES (?, ?, ?, ?, ?)]
[parameters: ('S-1-5-21-2720623347-3037847324-4167270909-1002', 'FEAR', None, '192.168.2.75', 'S-1-5-21-2720623347-3037847324-4167270909-1002')]
(Background on this error at: http://sqlalche.me/e/13/gkpj)""":
                self.session.rollback()
                user2 = User(**args)
                self.session.merge(user2)
                self.session.commit()
                """ self.session.rollback()
                self.session.close()
                self.session = self.create_Session()
                obj = self.query_user(f'uid == "{args["uid"]}"')[0]
                self.session.delete(obj)
                self.session.expire(obj)
                self.session.commit()
                del obj
                self.session.close()
                self.session = self.create_Session()
                return self._create(controller=self.UserTableController, **args)"""
            else:
                self.session.rollback()
                user2 = User(**args)
                self.session.merge(user2.CoT)
                self.session.commit()
                """print(e)
                self.session.rollback()
                self.session.close()
                self.session = self.create_Session()
                obj = self.query_CoT(f'uid == "{args["CoT"].uid}"')[0]
                self.session.delete(obj)
                self.session.commit()
                del obj
                return self._create(controller=self.UserTableController, **args)"""
        except Exception as e:
            print(e)
            try:
                self.session.rollback()
                self.session.close()
                self.session = self.create_Session()
                obj = self.query_CoT(f'uid == "{args["CoT"].uid}"')[0]
                self.session.delete(obj)
                self.session.commit()
                return self._create(controller=self.UserTableController, **args)
            except Exception as e:
                self.session.rollback()
                self.session.close()
                self.session = self.create_Session()
    def remove_user(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which datapackages are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.UserTableController, query=query)

    def query_user(self, query="1 == 1", column=['*']):
        return self._query(controller=self.UserTableController, query=query, columns=column)

    def update_user(self, column_value=dict(), query="1 == 1"):
        return self._update(controller=self.UserTableController, query=query, column_value=column_value)

    def create_systemUser(self, **args):
        try:
            return self._create(controller=self.SystemUserTableController, **args)
        except Exception as e:
            self.session.rollback()
            self.session.commit()
    def remove_systemUser(self, query="1 == 1"):
        return self._remove(controller=self.SystemUserTableController, query=query)

    def query_systemUser(self, query="1 == 1", column=['*']):
        return self._query(controller=self.SystemUserTableController, query=query, columns=column)

    def update_systemUser(self, column_value=dict(), query="1 == 1"):
        return self._update(controller=self.SystemUserTableController, query=query, column_value=column_value)

    def create_videostream(self, **args):
        return self._create(controller=self.VideoStreamTableController, **args)

    def remove_videostream(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which datapackages are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.VideoStreamTableController, query=query)

    def query_videostream(self, query="1 == 1", column=['*']):
        return self._query(controller=self.VideoStreamTableController, query=query, columns=column)

    def update_videostream(self, column_value=dict(), query="1 == 1"):
        return self._update(controller=self.VideoStreamTableController, query=query, column_value=column_value)

    def create_CoT(self, object):
        try:
            session = self.create_Session()
            output = self.EventTableController.create(session, object)
            session.close()
            return output
        except IntegrityError as e:
            try:
                session.rollback()
                session.close()
                obj = self.query_CoT(f'uid == "{object.uid}"')[0]
                newobj = self.EventTableController.convert_model_to_row(object)
                self.session.delete(obj)
                self.session.commit()
                make_transient(obj)
                newobj.User = obj.User

                self.session.add(newobj)
                self.session.commit()
            except Exception as e:
                raise e
        except Exception as e:
            self.session.rollback()
    def remove_CoT(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which datapackages are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.EventTableController, query=query)

    def query_CoT(self, query="1 == 1", column=['*']):
        return self._query(controller=self.EventTableController, query=query, columns=column)

    def update_CoT(self, column_value=dict(), query="1 == 1"):
        return self._update(controller=self.EventTableController, query=query, column_value=column_value)

    def create_APIUser(self, **args):
        return self._create(self.APIUserController, **args)

    def remove_APIUser(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which datapackages are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.APIUserController, query=query)

    def query_APIUser(self, query="1 == 1", column=['*']):
        return self._query(controller=self.APIUserController, query=query, columns=column)

    def update_APIUser(self, column_value=None, query="1 == 1"):
        return self._update(controller=self.APIUserController, query=query, column_value=column_value)

    def create_ActiveEmergency(self, object):
        try:
            session = self.create_Session()
            self.ActiveEmergencysController.create(session, object)
        except Exception as e:
            session.rollback()
            session.close()
    def remove_ActiveEmergency(self, query="1 == 1"):
        return self._remove(controller=self.ActiveEmergencysController, query=query)

    def query_ActiveEmergency(self, query="1 == 1", column=['*']):
        return self._query(controller=self.ActiveEmergencysController, query=query, columns=column)

    def update_ActiveEmergency(self, column_value=None, query="1 == 1"):
        return self._update(controller=self.ActiveEmergencysController, query=query, column_value=column_value)


    def create_ExCheck(self, object):
        try:
            session = self.create_Session()
            output = self.ExCheckController.create(session, object)
            session.close()
            return output
        except Exception as e:
            session.rollback()
            session.close()

    def remove_ExCheck(self, query="1 == 1"):
        return self._remove(controller=self.ExCheckController, query=query)

    def query_ExCheck(self, query = "1==1", column=["*"], verbose = False):
        if verbose == False:
            return self._query(controller=self.ExCheckController, query=query, columns=column)
        else:
            return self.ExCheckController.queryChildren(query, column, self.session)
    def update_ExCheck(self, query="1==1", column_value=None):
        return self._update(controller=self.ExCheckController, query=query, column_value=column_value)

    def create_Excheckchecklist(self, **args):
        return self._create(self.ExCheckChecklistController, **args)

    def remove_ExCheckChecklist(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which exCheckChecklists are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.ExCheckChecklistController, query=query)

    def query_ExCheckChecklist(self, query="1 == 1", column=['*']):
        return self._query(controller=self.ExCheckChecklistController, query=query, columns=column)

    def update_ExCheckChecklist(self, column_value=None, query="1 == 1"):
        return self._update(controller=self.ExCheckChecklistController, query=query, column_value=column_value)

    def create_ActiveFederation(self, **args):
        return self._create(self.ActiveFederationController, **args)

    def remove_ActiveFederation(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which exCheckChecklists are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.ActiveFederationController, query=query)

    def query_ActiveFederation(self, query="1 == 1", column=['*']):
        return self._query(controller=self.ActiveFederationController, query=query, columns=column)

    def update_ActiveFederation(self, column_value=None, query="1 == 1"):
        return self._update(controller=self.ActiveFederationController, query=query, column_value=column_value)

    def create_Federation(self, **args):
        return self._create(self.FederationController, **args)

    def remove_Federation(self, query="1 == 1"):
        '''
        :param query: this parameter will be used to select which exCheckChecklists are deleted
        :return: 1 on success
        '''
        return self._remove(controller=self.FederationController, query=query)

    def query_Federation(self, query="1 == 1", column=['*']):
        return self._query(controller=self.FederationController, query=query, columns=column)

    def update_Federation(self, column_value=None, query="1 == 1"):
        return self._update(controller=self.FederationController, query=query, column_value=column_value)

    def shutdown_Connection(self):
        self.session.close()
        self.engine.dispose()

if __name__ == "__main__":
    contr = DatabaseController()
    import datetime as dt
    #contr.create_datapackage(uid='9', CreatorUid='abc123', Hash='36e4506f4d6a9582fc60529525c55f3ebd42b887b5164d24e44b85d7ea686a3d', Keywords="foobar", MIMEType="aaa/bbb", Name="other", Privacy=1, Size=32, SubmissionDateTime=dt.datetime.utcnow(), SubmissionUser="blue")
    #contr.remove_APIUser(query='Username == "admin"')
    #contr.remove_datapackage('Hash == "36e4506f4d6a9582fc60529525c55f3ebd42b887b5164d24e44b85d7ea686a3d"')
    #UID = 123
    #callsign = contr.query_user(query=f'uid == "{UID}"', column=['callsign'])
    # contr.(uid='9', CreatorUid='abc123', Hash='1bc21o', Keywords="foobar", MIMEType="aaa/bbb", Name="other", Privacy=1, Size=32, SubmissionDateTime=dt.datetime.utcnow(), SubmissionUser="blue")
    y = contr.query_user()

    from FreeTAKServer.controllers.SpecificCoTControllers.SendDropPointController import SendDropPointController
    from FreeTAKServer.model.RawCoT import RawCoT
    from FreeTAKServer.model.SQLAlchemy.Event import Event
    from FreeTAKServer.model.SQLAlchemy.CoTTables.Detail import Detail
    import uuid
    m = RawCoT()
    e1 = Event(uid=str(uuid.uuid4()))
    d1 = Detail()
    d1.Event = e1
    contr.session.add(e1)
    contr.session.commit()
    m.xmlString = "<event version='2.0' uid='4oh2-4265fe-83hb-9bd3442e6de0' type='a-h-G' time='2020-05-06T13:29:25.146Z' start='2020-05-06T13:29:25.146Z' stale='2020-05-06T13:34:25.146Z' how='h-g-i-g-o'><point lat='44.22054348420014' lon='-66.14146066424053' hae='9999999.0' ce='9999999.0' le='9999999.0' /><detail><status readiness='true'/><archive/><precisionlocation altsrc='???'/><archive/><usericon iconsetpath='COT_MAPPING_2525B/a-h/a-h-G'/><contact callsign='Bob'/><color argb='-1'/><link uid='ANDROID-352614100054990' production_time='2020-01-20T00:15:03.281Z' type='a-f-G-U-C' parent_callsign='Peregrin' relation='p-p'/><marti><dest callsign = 'abc'/></marti></detail></event>"
    # m.xmlString = "<event version='2.0' uid='87f64797-4ae2-42ce-83cb-9bd3442e6dd0' type='a-h-G' time='2020-05-06T13:29:25.146Z' start='2020-05-06T13:29:25.146Z' stale='2020-05-06T13:34:25.146Z' how='h-g-i-g-o'><point lat='44.22054348420014' lon='-66.14146066424053' hae='9999999.0' ce='9999999.0' le='9999999.0' /><detail/></event>"
    x = SendDropPointController(m)
    b = RawCoT()
    b.dbController = contr
    b.xmlString = '<event version="2.0" uid="5bbka076-caea-4766-a300-1c9ac3f58d00" type="a-n-G" how="h-g-i-g-o" start="2020-09-30T00:56:29.71Z" time="2020-07-18T13:38:31.220Z" stale="2020-10-07T00:56:29.71Z"><detail><status readiness="true"/><usericon iconsetpath="COT_MAPPING_2525B/a-n/a-n-G"/><link uid="ANDROID-359975090666199" relation="p-p" production_time="2020-07-11T17:31:36.007Z" type="a-f-G-U-C" parent_callsign="NOVA"/><color argb="-1"/><precisionlocation altsrc="???"/><contact callsign="N.11.143136"/><remarks time="2020-09-30T00:56:12Z">DEFAULT</remarks></detail><point le="9999999.0" ce="9999999.0" hae="9999999.0" lon="-61.137312737751884" lat="-11.005735933018565"/></event>'
    x = SendDropPointController(b)
    y = x.getObject()
    contr.create_CoT(y.modelObject)
    x = contr.session.query(Event).all()[-1]
    #contr.create_user(callsign = 'abc')
    #import operator
    #a = contr.query_datapackage("uid == 423")



#######################################################
# 
# ReceiveConnections.py
# Python implementation of the Class ReceiveConnections
# Generated by Enterprise Architect
# Created on:      19-May-2020 6:21:05 PM
# Original author: Natha Paquette
# 
#######################################################
import socket
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.model.RawConnectionInformation import RawConnectionInformation as sat
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.controllers.configuration.ReceiveConnectionsConstants import ReceiveConnectionsConstants
loggingConstants = LoggingConstants()
logger = CreateLoggerController("ReceiveConnections").getLogger()
import logging
import logging.handlers
import ssl
import time
#TODO: move health check values to constants and create controller for HealthCheck data

class ReceiveConnections:
    def __init__(self):
        pass


    def listen(self, sock, sslstatus = False):
        #logger = CreateLoggerController("ReceiveConnections").getLogger()
        #listen for client connections
        sock.listen(0)
        try:
            #establish the socket variables
            if sslstatus == True:
                sock.settimeout(60)
                #sock.setblocking(0)
            logger.debug('receive connection started')
            try:
                client, address = sock.accept()
            except ssl.SSLError as e:
                logger.warning('ssl error thrown in connection attempt '+str(e))
                return -1
            if sslstatus == True:
                logger.info('client connected over ssl ' + str(address) + ' ' + str(time.time()))
            #wait to receive client
            client.settimeout(int(ReceiveConnectionsConstants().RECEIVECONNECTIONDATATIMEOUT))
            data = client.recv(1024)
            if data == ReceiveConnectionsConstants().TESTDATA:
                client.send(b'success')
                return -1
            else:
                pass
            client.settimeout(0)
            logger.info(loggingConstants.RECEIVECONNECTIONSLISTENINFO)
            #establish the socket array containing important information about the client
            RawConnectionInformation = sat()
            RawConnectionInformation.ip = address[0]
            RawConnectionInformation.socket = client
            RawConnectionInformation.xmlString = data.decode('utf-8')
            try:
                if socket != None and data != b'':
                    return RawConnectionInformation
                else:
                    return -1
            except Exception as e:
                logger.warning('exception in returning data '+str(e))
                return -1

        except Exception as e:
            logger.warning(loggingConstants.RECEIVECONNECTIONSLISTENERROR+str(e))
            return -1



#######################################################
# 
# Link.py
# Python implementation of the Class Link
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:33 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String, Integer
from sqlalchemy.orm import relationship


class Link(Base):
# default constructor  def __init__(self):  
    __tablename__ = "Link"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="link")
    #PrimaryKey = Column(ForeignKey('detail.uid'), primary_key=True)
    # Internet Media type of the referenced object.  If the link is to a CoT event,
    # the mime attribute is optional and its type may be application/xml or text/xml
    # as described in RFC 3023, "XML Media Types", or the unregistered type,
    # application/cot+xml.  If the link is to an arbitrary resource, the mime
    # attribute is required and and appropriate Internet media type must be specified.
    #  Registered media types are managed by the IANA and are listed at http://www.
    # iana.org/assignments/media-types/.
    mime = Column(String)
    parent_callsign = Column(String)
    production_time = Column(String)
    # The type of relationship (e.g, subject, object, indirect object) that this link
    # describes.  This is a hierarchy much like the event type field.
    relation = Column(String)
    # Remarks associated with this link.
    remarks = Column(String)
    # The CoT type of the referenced object.  This is included because it is
    # generally the key item needed in a tasking.
    type = Column(String)
    uid = Column(String)
    # If present, this is a URL through which the linked object can be retrieved.
    # If the URL is missing, then the object should be a periodic message (e.g., blue
    # force track) that can be read from a CoT stream.
    url = Column(String)
    # Version tag for this sub schema.  Neccessary to ensure upward compatibility
    # with future revisions.
    version = Column(String)


import os
currentPath = os.path.dirname(os.path.abspath(__file__))
from pathlib import Path


class MainConfig:
    """
    this is the main configuration file and is the only one which
    should need to be changed
    """
    # this is the port to which clients will connect
    CoTServicePort = int(8087)

    SSLCoTServicePort = int(8089)

    # this needs to be changed for private data packages to work
    DataPackageServiceDefaultIP = str("0.0.0.0")

    # User Connection package IP needs to be set to the IP which is used when creating the connection in your tak device
    UserConnectionIP = str("0.0.0.0")

    python_version = 'python3.8'

    userpath = '/usr/local/lib/'

    # api port
    APIPort = 19023

    # Federation port
    FederationPort = 9000

    # api IP
    APIIP = '0.0.0.0'

    # allowed ip's to access CLI commands
    AllowedCLIIPs = ['127.0.0.1']

    # IP for CLI to access
    CLIIP = '127.0.0.1'

    # whether or not to save CoT's to the DB
    SaveCoTToDB = bool(True)

    # this should be set before startup
    DBFilePath = str(r'/root/FTSDataBase.db')

    # the version information of the server (recommended to leave as default)
    version = 'FreeTAKServer-1.5.8 RC 1'

    MainPath = str(Path(fr'{userpath}{python_version}/dist-packages/FreeTAKServer'))

    ExCheckMainPath = str(Path(fr'{MainPath}/ExCheck'))

    ExCheckFilePath = str(Path(fr'{MainPath}/ExCheck/template'))

    ExCheckChecklistFilePath = str(Path(fr'{MainPath}/ExCheck/checklist'))

    DataPackageFilePath = str(Path(fr'{MainPath}/FreeTAKServerDataPackageFolder'))

    # format of API message header should be {Authentication: Bearer 'TOKEN'}
    from uuid import uuid4
    id = str(uuid4())

    nodeID = f"FreeTAKServer-{id}"

    # set to None if you don't want a message sent
    ConnectionMessage = f'Welcome to FreeTAKServer {version}. The Parrot is not dead. It’s just resting'

    keyDir = str(Path(fr'{MainPath}/certs/pubserver.key'))

    pemDir = str(Path(fr'{MainPath}/certs/pubserver.pem')) # or crt

    unencryptedKey = str(Path(fr'{MainPath}/certs/pubserver.key.unencrypted'))

    p12Dir = str(Path(fr'{MainPath}/certs/pubserver.p12'))

    CA = str(Path(fr'{MainPath}/certs/ca.pem'))
    CAkey = str(Path(fr'{MainPath}/certs/ca.key'))

    federationCert = str(Path(fr'{MainPath}/certs/pubserver.pem'))
    federationKey = str(Path(fr'{MainPath}/certs/pubserver.key'))
    federationKeyPassword = str('defaultpass')
    
    # location to backup client packages
    clientPackages = str(Path(fr'{MainPath}/certs/ClientPackages'))

    password = str('defaultpass')

    websocketkey = "YourWebsocketKey"


#######################################################
# 
# sendClientData.py
# Python implementation of the Class sendClientData
# Generated by Enterprise Architect
# Created on:      19-May-2020 7:37:17 PM
# Original author: Natha Paquette
# 
#######################################################

from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController

logger = CreateLoggerController("SendClientData").getLogger()

loggingConstants = LoggingConstants()

class SendClientData:
    def __init__(self):
        pass
    def SendDataInQueue(self,  dataQueue, clientQueue):
        logger.info(loggingConstants.SENDCLIENTDATASENDDATAINQUEUEINFO)
        while True:
            tempQueue = clientQueue
            tempArray = []
            if dataQueue.empty() == False:
                while tempQueue.empty() == False:
                   tempArray.append(tempQueue.get())
                data = dataQueue.get()
                sender = data.clientInformation
                for client in tempArray:
                    clientInformation = client.clientInformation
                    if clientInformation != sender:
                        sock = client.clientInformation.socket
                        
                        sock.send(data.xmlString)
                    else:
                        pass
            else:
                pass
    
    def HelloWorld(self):
        print('hello world')
        '''
        killSwitch = 0
        try:
            while killSwitch == 0:
                time.sleep(const.DELAY)
                if killSwitch == 1:
                    break
                if len(self.emergencyDict)>0:
                        for x in self.emergencyDict:
                            client.send(self.emergencyDict[x])
                        logger.debug('emergency activated')
                else:
                    pass

                if len(self.client_dict[current_id]['main_data'])>0:

                    for x in self.client_dict[current_id]['main_data']:
                        logger.debug(self.client_dict[current_id]['main_data'])
                        client.send(x)
                        print('\n'+'sent '+ str(x)+' to '+ str(address) + '\n')
                        self.client_dict[current_id]['main_data'].remove(x)

                else:
                    client.send(Serializer().serializerRoot(RequestCOTController().ping(eventuid = uuid.uuid1())).encode())
            client.shutdown()
            client.close()
        except Exception as e:
            logger.warning('error in send info '+str(e))
            client.shutdown()
            client.close()
            return 1
        '''


from FreeTAKServer.controllers.services.Orchestrator import Orchestrator
from FreeTAKServer.controllers.ClientReceptionHandler import ClientReceptionHandler
from FreeTAKServer.controllers.ReceiveConnections import ReceiveConnections
from FreeTAKServer.controllers.TCPSocketController import TCPSocketController
import os
from multiprocessing.pool import ThreadPool
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
loggingConstants = LoggingConstants()
logger = CreateLoggerController("FTS").getLogger()

class TCPCoTServiceController(Orchestrator):
    def start(self, IP, CoTPort, Event, clientDataPipe, ReceiveConnectionKillSwitch, RestAPIPipe):
        try:
            self.dbController = DatabaseController()
            # self.clear_user_table()
            os.chdir('../../../')
            # create socket controller
            self.TCPSocketController = TCPSocketController()
            self.TCPSocketController.changeIP(IP)
            self.TCPSocketController.changePort(CoTPort)
            sock = self.TCPSocketController.createSocket()
            pool = ThreadPool(processes=2)
            self.pool = pool
            clientData = pool.apply_async(ClientReceptionHandler().startup, (self.clientInformationQueue,))
            receiveConnection = pool.apply_async(ReceiveConnections().listen, (sock,))
            # instantiate domain model and save process as object
            self.mainRunFunction(clientData, receiveConnection, sock, pool, Event, clientDataPipe,
                                 ReceiveConnectionKillSwitch, RestAPIPipe)
        except Exception as e:
            logger.error('there has been an exception in the start function '
                         'of TCPCoTService ' + str(e))
            return e



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.SummaryVariables import SummaryVariables as vars

class Summary(FTSProtocolObject):
    def __init__(self):
        self.INTAG = vars().INTAG

    @staticmethod
    def drop_point(INTAG = vars.drop_point().INTAG):
        summary = Summary()
        summary.INTAG = INTAG
        return summary

    def setINTAG(self, INTAG = None):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG


from FreeTAKServer.controllers.MainSocketController import MainSocketController
from FreeTAKServer.model.sockets.SSLServerSocket import SSLServerSocket
import ssl
import socket


class SSLSocketController(MainSocketController):
    def __init__(self):
        self.MainSocket = SSLServerSocket()

    def createSocket(self):
        context = ssl.SSLContext(protocol=ssl.PROTOCOL_TLS_SERVER)
        context.load_verify_locations(cafile=self.MainSocket.CA)
        context.options |= ssl.OP_NO_SSLv3
        context.options |= ssl.OP_NO_SSLv2
        context.verify_mode = ssl.CERT_REQUIRED
        context.load_cert_chain(certfile=self.MainSocket.pemDir, keyfile=self.MainSocket.keyDir,
                                password=self.MainSocket.password, )
        self.MainSocket.sock = socket.socket(self.MainSocket.socketAF, self.MainSocket.socketSTREAM)
        self.MainSocket.sock.setsockopt(self.MainSocket.solSock, self.MainSocket.soReuseAddr,
                                        self.MainSocket.sockProto)
        self.MainSocket.sock.bind((self.MainSocket.ip, self.MainSocket.port))
        self.MainSocket.sock = context.wrap_socket(self.MainSocket.sock, server_side=True)
        return self.MainSocket.sock

    def createClientSocket(self, serverIP):
        context = ssl.SSLContext()
        context.load_verify_locations(cafile=self.MainSocket.CA)
        context.load_cert_chain(certfile=self.MainSocket.pemDir, keyfile=self.MainSocket.keyDir,
                                password=self.MainSocket.password, )
        context.verify_mode = ssl.CERT_OPTIONAL
        context.check_hostname = False
        context.set_ciphers('DEFAULT@SECLEVEL=1')
        self.MainSocket.sock = socket.socket(self.MainSocket.socketAF, self.MainSocket.socketSTREAM)
        self.MainSocket.sock = context.wrap_socket(self.MainSocket.sock)
        return self.MainSocket.sock


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# Status.py
# Python implementation of the Class Status
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):07 AM
# Original author: Corvo
# 
#######################################################
from FreeTAKServer.model.FTSModelVariables.statusVariables import statusVariables as vars

class Status(FTSProtocolObject):
    def __init__(self):
        self.battery = vars().BATTERY
        self.readiness = vars().READINESS
    # % of the battery on the phone

    @staticmethod
    def drop_point(battery=vars.drop_point().BATTERY, readiness=vars.drop_point().READINESS):
        status = Status()
        status.setbattery(battery=battery)
        status.setreadiness(readiness=readiness)
        return status

    @staticmethod
    def connection(battery=vars.connection().BATTERY, readiness=vars.connection().READINESS):
        status = Status()
        status.setbattery(battery=battery)
        status.setreadiness(readiness=readiness)
        return status

    @staticmethod
    def FederatedCoT():
        status = Status()
        return status

    @staticmethod
    def UserUpdate(BATTERY=vars.UserUpdate().battery):
        status = Status()
        status.setbattery(BATTERY)
        return status

    # battery getter 
    def getbattery(self): 
        return self.battery 

    # battery setter 
    def setbattery(self, battery=0):  
        self.battery = battery

    # probably boolean to determine if ready or not
     
    # readiness getter 
    def getreadiness(self): 
        return self.readiness 

    # readiness setter 
    def setreadiness(self, readiness=0):  
        self.readiness = readiness


if __name__ == "__main__":
    Status()


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fig.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='fig.proto',
  package='com.atakmap',
  syntax='proto3',
  serialized_options=b'\242\002\003TAK',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\tfig.proto\x12\x0b\x63om.atakmap\"~\n\x0e\x46\x65\x64\x65ratedEvent\x12$\n\x05\x65vent\x18\x01 \x01(\x0b\x32\x15.com.atakmap.GeoEvent\x12.\n\x07\x63ontact\x18\x02 \x01(\x0b\x32\x1d.com.atakmap.ContactListEntry\x12\x16\n\x0e\x66\x65\x64\x65rateGroups\x18\x03 \x03(\t\"\xa5\x03\n\x08GeoEvent\x12\x10\n\x08sendTime\x18\x01 \x01(\x03\x12\x11\n\tstartTime\x18\x02 \x01(\x03\x12\x11\n\tstaleTime\x18\x03 \x01(\x03\x12\x0b\n\x03lat\x18\x04 \x01(\x01\x12\x0b\n\x03lon\x18\x05 \x01(\x01\x12\x0b\n\x03hae\x18\x06 \x01(\x01\x12\n\n\x02\x63\x65\x18\x07 \x01(\x01\x12\n\n\x02le\x18\x08 \x01(\x01\x12\x0b\n\x03uid\x18\t \x01(\t\x12\x0c\n\x04type\x18\n \x01(\t\x12\x13\n\x0b\x63oordSource\x18\x0b \x01(\t\x12\r\n\x05other\x18\x0c \x01(\t\x12\x0f\n\x07\x62\x61ttery\x18\r \x01(\x05\x12\x0c\n\x04ploc\x18\x0e \x01(\t\x12\x0c\n\x04palt\x18\x0f \x01(\t\x12\x12\n\nscreenName\x18\x10 \x01(\t\x12\x11\n\tgroupName\x18\x11 \x01(\t\x12\x11\n\tgroupRole\x18\x12 \x01(\t\x12\r\n\x05phone\x18\x13 \x01(\t\x12\r\n\x05speed\x18\x14 \x01(\x01\x12\x0e\n\x06\x63ourse\x18\x15 \x01(\x01\x12\'\n\x06\x62inary\x18\x16 \x01(\x0b\x32\x17.com.atakmap.BinaryBlob\x12\x0f\n\x07ptpUids\x18\x17 \x03(\t\x12\x14\n\x0cptpCallsigns\x18\x18 \x03(\t\"}\n\nBinaryBlob\x12\'\n\x04type\x18\x01 \x01(\x0e\x32\x19.com.atakmap.BINARY_TYPES\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x10\n\x08\x66ilename\x18\x03 \x01(\t\x12\x11\n\ttimestamp\x18\x04 \x01(\x03\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\"\x8a\x01\n\x10\x43ontactListEntry\x12$\n\toperation\x18\x01 \x01(\x0e\x32\x11.com.atakmap.CRUD\x12\x0b\n\x03uid\x18\x02 \x01(\t\x12\x10\n\x08\x63\x61llsign\x18\x03 \x01(\t\x12\r\n\x05phone\x18\x04 \x01(\t\x12\x0b\n\x03sip\x18\x05 \x01(\t\x12\x15\n\rdirectConnect\x18\x06 \x01(\t\"\x07\n\x05\x45mpty\":\n\x08Identity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03uid\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\"G\n\x0cSubscription\x12\'\n\x08identity\x18\x01 \x01(\x0b\x32\x15.com.atakmap.Identity\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\"\x83\x01\n\x0c\x43lientHealth\x12\x37\n\x06status\x18\x01 \x01(\x0e\x32\'.com.atakmap.ClientHealth.ServingStatus\":\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\"\x96\x01\n\x0cServerHealth\x12\x37\n\x06status\x18\x01 \x01(\x0e\x32\'.com.atakmap.ServerHealth.ServingStatus\"M\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x12\x11\n\rNOT_CONNECTED\x10\x03\"@\n\x03ROL\x12\x0f\n\x07program\x18\x01 \x01(\t\x12(\n\x07payload\x18\x02 \x03(\x0b\x32\x17.com.atakmap.BinaryBlob*A\n\x04\x43RUD\x12\x0b\n\x07INVALID\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\x08\n\x04READ\x10\x02\x12\n\n\x06UPDATE\x10\x03\x12\n\n\x06\x44\x45LETE\x10\x04*/\n\x0c\x42INARY_TYPES\x12\t\n\x05\x45MPTY\x10\x00\x12\t\n\x05OTHER\x10\x01\x12\t\n\x05IMAGE\x10\x02\x32\x88\x05\n\x10\x46\x65\x64\x65ratedChannel\x12\x41\n\x0cSendOneEvent\x12\x1b.com.atakmap.FederatedEvent\x1a\x12.com.atakmap.Empty\"\x00\x12\x46\n\x13\x42inaryMessageStream\x12\x17.com.atakmap.BinaryBlob\x1a\x12.com.atakmap.Empty\"\x00(\x01\x12<\n\x0bSendOneBlob\x12\x17.com.atakmap.BinaryBlob\x1a\x12.com.atakmap.Empty\"\x00\x12:\n\x0bgetIdentity\x12\x12.com.atakmap.Empty\x1a\x15.com.atakmap.Identity\"\x00\x12O\n\x11\x43lientEventStream\x12\x19.com.atakmap.Subscription\x1a\x1b.com.atakmap.FederatedEvent\"\x00\x30\x01\x12O\n\x11ServerEventStream\x12\x1b.com.atakmap.FederatedEvent\x1a\x19.com.atakmap.Subscription\"\x00(\x01\x12\x45\n\x0bHealthCheck\x12\x19.com.atakmap.ClientHealth\x1a\x19.com.atakmap.ServerHealth\"\x00\x12\x42\n\x0f\x43lientROLStream\x12\x19.com.atakmap.Subscription\x1a\x10.com.atakmap.ROL\"\x00\x30\x01\x12\x42\n\x0fServerROLStream\x12\x10.com.atakmap.ROL\x1a\x19.com.atakmap.Subscription\"\x00(\x01\x42\x06\xa2\x02\x03TAKb\x06proto3'
)

_CRUD = _descriptor.EnumDescriptor(
  name='CRUD',
  full_name='com.atakmap.CRUD',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INVALID', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='READ', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DELETE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1341,
  serialized_end=1406,
)
_sym_db.RegisterEnumDescriptor(_CRUD)

CRUD = enum_type_wrapper.EnumTypeWrapper(_CRUD)
_BINARY_TYPES = _descriptor.EnumDescriptor(
  name='BINARY_TYPES',
  full_name='com.atakmap.BINARY_TYPES',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EMPTY', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OTHER', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IMAGE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1408,
  serialized_end=1455,
)
_sym_db.RegisterEnumDescriptor(_BINARY_TYPES)

BINARY_TYPES = enum_type_wrapper.EnumTypeWrapper(_BINARY_TYPES)
INVALID = 0
CREATE = 1
READ = 2
UPDATE = 3
DELETE = 4
EMPTY = 0
OTHER = 1
IMAGE = 2


_CLIENTHEALTH_SERVINGSTATUS = _descriptor.EnumDescriptor(
  name='ServingStatus',
  full_name='com.atakmap.ClientHealth.ServingStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SERVING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_SERVING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1062,
  serialized_end=1120,
)
_sym_db.RegisterEnumDescriptor(_CLIENTHEALTH_SERVINGSTATUS)

_SERVERHEALTH_SERVINGSTATUS = _descriptor.EnumDescriptor(
  name='ServingStatus',
  full_name='com.atakmap.ServerHealth.ServingStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SERVING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_SERVING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_CONNECTED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1196,
  serialized_end=1273,
)
_sym_db.RegisterEnumDescriptor(_SERVERHEALTH_SERVINGSTATUS)


_FEDERATEDEVENT = _descriptor.Descriptor(
  name='FederatedEvent',
  full_name='com.atakmap.FederatedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event', full_name='com.atakmap.FederatedEvent.event', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contact', full_name='com.atakmap.FederatedEvent.contact', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='federateGroups', full_name='com.atakmap.FederatedEvent.federateGroups', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=152,
)


_GEOEVENT = _descriptor.Descriptor(
  name='GeoEvent',
  full_name='com.atakmap.GeoEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sendTime', full_name='com.atakmap.GeoEvent.sendTime', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='startTime', full_name='com.atakmap.GeoEvent.startTime', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='staleTime', full_name='com.atakmap.GeoEvent.staleTime', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lat', full_name='com.atakmap.GeoEvent.lat', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lon', full_name='com.atakmap.GeoEvent.lon', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hae', full_name='com.atakmap.GeoEvent.hae', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ce', full_name='com.atakmap.GeoEvent.ce', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='le', full_name='com.atakmap.GeoEvent.le', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='com.atakmap.GeoEvent.uid', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='com.atakmap.GeoEvent.type', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='coordSource', full_name='com.atakmap.GeoEvent.coordSource', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='other', full_name='com.atakmap.GeoEvent.other', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='battery', full_name='com.atakmap.GeoEvent.battery', index=12,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ploc', full_name='com.atakmap.GeoEvent.ploc', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='palt', full_name='com.atakmap.GeoEvent.palt', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='screenName', full_name='com.atakmap.GeoEvent.screenName', index=15,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='groupName', full_name='com.atakmap.GeoEvent.groupName', index=16,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='groupRole', full_name='com.atakmap.GeoEvent.groupRole', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phone', full_name='com.atakmap.GeoEvent.phone', index=18,
      number=19, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='speed', full_name='com.atakmap.GeoEvent.speed', index=19,
      number=20, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='course', full_name='com.atakmap.GeoEvent.course', index=20,
      number=21, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='binary', full_name='com.atakmap.GeoEvent.binary', index=21,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ptpUids', full_name='com.atakmap.GeoEvent.ptpUids', index=22,
      number=23, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ptpCallsigns', full_name='com.atakmap.GeoEvent.ptpCallsigns', index=23,
      number=24, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=576,
)


_BINARYBLOB = _descriptor.Descriptor(
  name='BinaryBlob',
  full_name='com.atakmap.BinaryBlob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='com.atakmap.BinaryBlob.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='com.atakmap.BinaryBlob.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='filename', full_name='com.atakmap.BinaryBlob.filename', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='com.atakmap.BinaryBlob.timestamp', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='com.atakmap.BinaryBlob.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=578,
  serialized_end=703,
)


_CONTACTLISTENTRY = _descriptor.Descriptor(
  name='ContactListEntry',
  full_name='com.atakmap.ContactListEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='com.atakmap.ContactListEntry.operation', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='com.atakmap.ContactListEntry.uid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='callsign', full_name='com.atakmap.ContactListEntry.callsign', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phone', full_name='com.atakmap.ContactListEntry.phone', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sip', full_name='com.atakmap.ContactListEntry.sip', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='directConnect', full_name='com.atakmap.ContactListEntry.directConnect', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=706,
  serialized_end=844,
)


_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='com.atakmap.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=846,
  serialized_end=853,
)


_IDENTITY = _descriptor.Descriptor(
  name='Identity',
  full_name='com.atakmap.Identity',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='com.atakmap.Identity.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='com.atakmap.Identity.uid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='com.atakmap.Identity.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=855,
  serialized_end=913,
)


_SUBSCRIPTION = _descriptor.Descriptor(
  name='Subscription',
  full_name='com.atakmap.Subscription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='identity', full_name='com.atakmap.Subscription.identity', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='filter', full_name='com.atakmap.Subscription.filter', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=915,
  serialized_end=986,
)


_CLIENTHEALTH = _descriptor.Descriptor(
  name='ClientHealth',
  full_name='com.atakmap.ClientHealth',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='com.atakmap.ClientHealth.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CLIENTHEALTH_SERVINGSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=989,
  serialized_end=1120,
)


_SERVERHEALTH = _descriptor.Descriptor(
  name='ServerHealth',
  full_name='com.atakmap.ServerHealth',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='com.atakmap.ServerHealth.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SERVERHEALTH_SERVINGSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1123,
  serialized_end=1273,
)


_ROL = _descriptor.Descriptor(
  name='ROL',
  full_name='com.atakmap.ROL',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='program', full_name='com.atakmap.ROL.program', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='com.atakmap.ROL.payload', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1275,
  serialized_end=1339,
)

_FEDERATEDEVENT.fields_by_name['event'].message_type = _GEOEVENT
_FEDERATEDEVENT.fields_by_name['contact'].message_type = _CONTACTLISTENTRY
_GEOEVENT.fields_by_name['binary'].message_type = _BINARYBLOB
_BINARYBLOB.fields_by_name['type'].enum_type = _BINARY_TYPES
_CONTACTLISTENTRY.fields_by_name['operation'].enum_type = _CRUD
_SUBSCRIPTION.fields_by_name['identity'].message_type = _IDENTITY
_CLIENTHEALTH.fields_by_name['status'].enum_type = _CLIENTHEALTH_SERVINGSTATUS
_CLIENTHEALTH_SERVINGSTATUS.containing_type = _CLIENTHEALTH
_SERVERHEALTH.fields_by_name['status'].enum_type = _SERVERHEALTH_SERVINGSTATUS
_SERVERHEALTH_SERVINGSTATUS.containing_type = _SERVERHEALTH
_ROL.fields_by_name['payload'].message_type = _BINARYBLOB
DESCRIPTOR.message_types_by_name['FederatedEvent'] = _FEDERATEDEVENT
DESCRIPTOR.message_types_by_name['GeoEvent'] = _GEOEVENT
DESCRIPTOR.message_types_by_name['BinaryBlob'] = _BINARYBLOB
DESCRIPTOR.message_types_by_name['ContactListEntry'] = _CONTACTLISTENTRY
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['Identity'] = _IDENTITY
DESCRIPTOR.message_types_by_name['Subscription'] = _SUBSCRIPTION
DESCRIPTOR.message_types_by_name['ClientHealth'] = _CLIENTHEALTH
DESCRIPTOR.message_types_by_name['ServerHealth'] = _SERVERHEALTH
DESCRIPTOR.message_types_by_name['ROL'] = _ROL
DESCRIPTOR.enum_types_by_name['CRUD'] = _CRUD
DESCRIPTOR.enum_types_by_name['BINARY_TYPES'] = _BINARY_TYPES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FederatedEvent = _reflection.GeneratedProtocolMessageType('FederatedEvent', (_message.Message,), {
  'DESCRIPTOR' : _FEDERATEDEVENT,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.FederatedEvent)
  })
_sym_db.RegisterMessage(FederatedEvent)

GeoEvent = _reflection.GeneratedProtocolMessageType('GeoEvent', (_message.Message,), {
  'DESCRIPTOR' : _GEOEVENT,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.GeoEvent)
  })
_sym_db.RegisterMessage(GeoEvent)

BinaryBlob = _reflection.GeneratedProtocolMessageType('BinaryBlob', (_message.Message,), {
  'DESCRIPTOR' : _BINARYBLOB,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.BinaryBlob)
  })
_sym_db.RegisterMessage(BinaryBlob)

ContactListEntry = _reflection.GeneratedProtocolMessageType('ContactListEntry', (_message.Message,), {
  'DESCRIPTOR' : _CONTACTLISTENTRY,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.ContactListEntry)
  })
_sym_db.RegisterMessage(ContactListEntry)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.Empty)
  })
_sym_db.RegisterMessage(Empty)

Identity = _reflection.GeneratedProtocolMessageType('Identity', (_message.Message,), {
  'DESCRIPTOR' : _IDENTITY,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.Identity)
  })
_sym_db.RegisterMessage(Identity)

Subscription = _reflection.GeneratedProtocolMessageType('Subscription', (_message.Message,), {
  'DESCRIPTOR' : _SUBSCRIPTION,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.Subscription)
  })
_sym_db.RegisterMessage(Subscription)

ClientHealth = _reflection.GeneratedProtocolMessageType('ClientHealth', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTHEALTH,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.ClientHealth)
  })
_sym_db.RegisterMessage(ClientHealth)

ServerHealth = _reflection.GeneratedProtocolMessageType('ServerHealth', (_message.Message,), {
  'DESCRIPTOR' : _SERVERHEALTH,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.ServerHealth)
  })
_sym_db.RegisterMessage(ServerHealth)

ROL = _reflection.GeneratedProtocolMessageType('ROL', (_message.Message,), {
  'DESCRIPTOR' : _ROL,
  '__module__' : 'fig_pb2'
  # @@protoc_insertion_point(class_scope:com.atakmap.ROL)
  })
_sym_db.RegisterMessage(ROL)


DESCRIPTOR._options = None

_FEDERATEDCHANNEL = _descriptor.ServiceDescriptor(
  name='FederatedChannel',
  full_name='com.atakmap.FederatedChannel',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1458,
  serialized_end=2106,
  methods=[
  _descriptor.MethodDescriptor(
    name='SendOneEvent',
    full_name='com.atakmap.FederatedChannel.SendOneEvent',
    index=0,
    containing_service=None,
    input_type=_FEDERATEDEVENT,
    output_type=_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='BinaryMessageStream',
    full_name='com.atakmap.FederatedChannel.BinaryMessageStream',
    index=1,
    containing_service=None,
    input_type=_BINARYBLOB,
    output_type=_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SendOneBlob',
    full_name='com.atakmap.FederatedChannel.SendOneBlob',
    index=2,
    containing_service=None,
    input_type=_BINARYBLOB,
    output_type=_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getIdentity',
    full_name='com.atakmap.FederatedChannel.getIdentity',
    index=3,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_IDENTITY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClientEventStream',
    full_name='com.atakmap.FederatedChannel.ClientEventStream',
    index=4,
    containing_service=None,
    input_type=_SUBSCRIPTION,
    output_type=_FEDERATEDEVENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ServerEventStream',
    full_name='com.atakmap.FederatedChannel.ServerEventStream',
    index=5,
    containing_service=None,
    input_type=_FEDERATEDEVENT,
    output_type=_SUBSCRIPTION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='com.atakmap.FederatedChannel.HealthCheck',
    index=6,
    containing_service=None,
    input_type=_CLIENTHEALTH,
    output_type=_SERVERHEALTH,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClientROLStream',
    full_name='com.atakmap.FederatedChannel.ClientROLStream',
    index=7,
    containing_service=None,
    input_type=_SUBSCRIPTION,
    output_type=_ROL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ServerROLStream',
    full_name='com.atakmap.FederatedChannel.ServerROLStream',
    index=8,
    containing_service=None,
    input_type=_ROL,
    output_type=_SUBSCRIPTION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_FEDERATEDCHANNEL)

DESCRIPTOR.services_by_name['FederatedChannel'] = _FEDERATEDCHANNEL

# @@protoc_insertion_point(module_scope)



from FreeTAKServer.model.RestMessages.Presence import Presence, RestEnumerations

class PresencePost(Presence):
    def __init__(self):
        pass
    
    def sethow(self, how):
        self.how = RestEnumerations.how[how]
    
    def gethow(self):
        return self.how
    
    def setlatitude(self, latitude):
        self.latitude = str(latitude)

    def getlatitude(self):
        return self.latitude

    def setlongitude(self, longitude):
        self.longitude = str(longitude)

    def getlongitude(self):
        return self.longitude
    
    def setname(self, name):
        self.name = str(name)

    def getname(self):
        return self.name
    
    def setrole(self, role):
        self.role = RestEnumerations.roles[role]

    def getrole(self):
        return self.role
    
    def setteam(self, team):
        self.team = RestEnumerations.Teams[team]

    def getteam(self):
        return self.team

    def settimeout(self, timeout):
        self.timeout = timeout

    def gettimeout(self):
        return self.timeout


from FreeTAKServer.controllers.DatabaseControllers.table_controllers import TableController
from FreeTAKServer.model.SQLAlchemy.ExCheckChecklist import ExCheckChecklist

class ExCheckChecklistController(TableController):
    def __init__(self):
        self.table = ExCheckChecklist


import http.client
import json
import tabulate
from FreeTAKServer.controllers.configuration.RestAPIVariables import RestAPIVariables as vars
from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.model.ServiceObjects.FTS import FTS
from FreeTAKServer.model.ServiceObjects.RestAPIService import RestAPIService
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

loggingConstants = LoggingConstants()
logger = CreateLoggerController("CLI").getLogger()

json_content = vars()
json_content.default_values()
json_content.json_content()
connectionIP = MainConfig.CLIIP
connectionPort = int(RestAPIService().RestAPIServicePort)

#TODO: standardize and abstract this
class RestCLIClient:

    def __init__(self):
        self.killSwitch = False
        self.conn = http.client.HTTPConnection(connectionIP, connectionPort)

    def check_response(self, response):
        if response.code == 200:
            return True
        else:
            return False
    def change_connection_info(self):
        global connectionIP, connectionPort
        ip = str(input('please enter desired IP for connection[' + str(connectionIP) + ']') or connectionIP)
        port = int(input('please enter desired Port for connection[' + str(connectionPort) + ']') or connectionPort)
        connectionIP = ip
        connectionPort = port
        print('connection information changed')
        return 1
    def start_CoT_service(self):
        try:
            self.CoTIP = str(
                input('enter CoT_service IP[' + str(FTS().CoTService.CoTServiceIP) + ']: ')) or FTS().CoTService.CoTServiceIP
            self.CoTPort = input('enter CoT_service Port[' + str(FTS().CoTService.CoTServicePort) + ']: ') or int(
                FTS().CoTService.CoTServicePort)
            self.CoTPort = int(self.CoTPort)
            #send start COT
            json_content.setdefaultCoTPort(self.CoTPort)
            json_content.setdefaultCoTIP(self.CoTIP)
            json_content.setdefaultCoTStatus('start')
            body = json.dumps(json_content.getJsonStatusStartAll())
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus",  body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('CoT service started')
            else:
                print('CoT service startup failed')
            return 1
        except Exception as e:
            logger.error('an exception has been thrown in CoT service startup ' + str(e))
            return -1

    def start_SSL_CoT_service(self):
        try:
            self.SSLCoTIP = str(
                input('enter SSL_CoT_service IP[' + str(FTS().SSLCoTService.SSLCoTServiceIP) + ']: ')) or FTS().SSLCoTService.SSLCoTServiceIP
            self.SSLCoTPort = input('enter SSL_CoT_service Port[' + str(FTS().SSLCoTService.SSLCoTServicePort) + ']: ') or int(
                FTS().SSLCoTService.SSLCoTServicePort)
            self.SSLCoTPort = int(self.SSLCoTPort)
            json_content.setdefaultSSLCoTPort(self.SSLCoTPort)
            json_content.setdefaultSSLCoTIP(self.SSLCoTIP)
            json_content.setdefaultSSLCoTStatus('start')
            body = json.dumps(json_content.getJsonStatusStartAll())
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('CoT service started')
            else:
                print('CoT service startup failed')
            return 1

        except Exception as e:
            logger.error('an exception has been thrown in CoT service startup ' + str(e))
            return -1

    def stop_CoT_service(self):
        try:
            json_content.setdefaultCoTStatus('stop')
            body = json.dumps(json_content.getJsonStatusStartAll())
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('CoT service stopped')
            else:
                print('CoT service stopping failed')
            return 1
        except Exception as e:
            logger.error("there's been an exception in the stopping of CoT Service " + str(e))
            return -1

    def stop_SSL_CoT_service(self):
        try:
            json_content.setdefaultSSLCoTStatus('stop')
            body = json.dumps(json_content.getJsonStatusStartAll())
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('CoT service stopped')
            else:
                print('CoT service stopping failed')
            return 1
        except Exception as e:
            logger.error("there's been an exception in the stopping of CoT Service " + str(e))
            return -1

    def start_data_package_service(self):
        try:

            self.APIPort = str(input('enter DataPackage_Service Port[' + str(
                DataPackageServerConstants().APIPORT) + ']: ')) or DataPackageServerConstants().APIPORT
            self.APIPort = int(self.APIPort)
            self.APIIP = str(input('enter DataPackage_Service IP[' + str(
                DataPackageServerConstants().IP) + ']: ')) or DataPackageServerConstants().IP
            #send start request
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            body = json.dumps({"TCPDataPackageService":{"IP": str(self.APIIP), "PORT": int(self.APIPort)}})
            conn.request("POST", "/TCPDataPackageService", body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('data package service started')
            else:
                print('data package service startup failed')
            return 1
        except Exception as e:
            logger.error('there has been an exception in the indevidual starting of the Dwata Packages Service')
            return -1

    def stop_data_package_service(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("DELETE", "/TCPDataPackageService")
            response = conn.getresponse()
            if self.check_response(response):
                print('data package service stoped')
            else:
                print('data package service stop failed')
            return 1
        except Exception as e:
            logger.error("there's been an exception in the termination of DataPackage Service " + str(e))
            return -1

    def start_federation_client_service(self):
        try:
            json_content = vars()
            json_content.default_values()
            json_content.json_content()


            json_content.setdefaultFederationClientIP(input('enter FederationClientService IP[' + str(
                FTS().FederationClientService.FederationClientServiceIP) + ']: ') or FTS().FederationClientService.FederationClientServiceIP)
            json_content.setdefaultFederationClientPort(
                input('enter FederationClientService Port[' + str(FTS().FederationClientService.FederationClientServicePort) + ']: ') or int(
                    FTS().FederationClientService.FederationClientServicePort))
            json_content.setdefaultFederationClientStatus('start')
            body = json.dumps(json_content.getJsonStatusStartAll())
            # send start request
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body,
                         {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('federation client service started')
            else:
                print('federation client service start failed')
            return 1
        except Exception as e:
            return -1

    def stop_federation_client_service(self):
        try:
            json_content = vars()
            json_content.default_values()
            json_content.json_content()
            json_content.setdefaultFederationClientStatus('stop')

            body = json.dumps(json_content.getJsonStatusStartAll())
            # send start request
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body,
                         {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('federation client service started')
            else:
                print('federation client service start failed')
            return 1
        except Exception as e:
            return -1
    def start_federation_server_service(self):
        try:
            json_content = vars()
            json_content.default_values()
            json_content.json_content()


            json_content.setdefaultFederationServerIP(input('enter FederationServerService IP[' + str(
                FTS().FederationServerService.FederationServerServiceIP) + ']: ') or FTS().FederationServerService.FederationServerServiceIP)
            json_content.setdefaultFederationServerPort(
                input('enter FederationServerService Port[' + str(FTS().FederationServerService.FederationServerServicePort) + ']: ') or int(
                    FTS().FederationServerService.FederationServerServicePort))
            json_content.setdefaultFederationServerStatus('start')
            body = json.dumps(json_content.getJsonStatusStartAll())
            # send start request
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body,
                         {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('federation server service started')
            else:
                print('federation server service start failed')
            return 1
        except Exception as e:
            return -1

    def stop_federation_server_service(self):
        try:
            json_content = vars()
            json_content.json_content()
            json_content.default_values()
            json_content.setdefaultFederationServerStatus('stop')

            body = json.dumps(json_content.getJsonStatusStartAll())
            # send start request
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/TCPDataPackageService", body,
                         {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('federation server service started')
            else:
                print('federation server service start failed')
            return 1
        except Exception as e:
            return -1

    def start_all(self):
        try:
            json_content.setdefaultCoTIP(input('Please enter the CoT service IP [' + str(FTS().CoTService.CoTServiceIP) + ']: ') or FTS().CoTService.CoTServiceIP)
            json_content.setdefaultCoTPort(input('Please enter the CoT service Port [' + str(FTS().CoTService.CoTServicePort) + ']: ') or FTS().CoTService.CoTServicePort)
            json_content.setdefaultDataPackagePort(input('Please enter the Data Package service Port [' + str(FTS().TCPDataPackageService.TCPDataPackageServicePort) + ']: ') or FTS().TCPDataPackageService.TCPDataPackageServicePort)
            json_content.setdefaultDataPackageIP(input('Please enter the Data Package service IP [' + str(FTS().TCPDataPackageService.TCPDataPackageServiceIP) + ']: ') or FTS().TCPDataPackageService.TCPDataPackageServiceIP)
            json_content.setdefaultSSLCoTIP(input('enter SSL_CoT_service IP[' + str(FTS().SSLCoTService.SSLCoTServiceIP) + ']: ') or FTS().SSLCoTService.SSLCoTServiceIP)
            json_content.setdefaultSSLCoTPort(input('enter SSL_CoT_service Port[' + str(FTS().SSLCoTService.SSLCoTServicePort) + ']: ') or int(FTS().SSLCoTService.SSLCoTServicePort))
            body = json.dumps(json_content.getJsonStatusStartAll())
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("POST", "/changeStatus", body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            conn.close()
            if self.check_response(response):
                print('server started')
            else:
                print('server startup failed')
            return 1
        except Exception as e:
            logger.error('there has been an exception in RestCLIClient start_all ' + str(e))
            return -1
    
    def stop_all(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            body = json.dumps({"CoTService": {"STATUS": "stop"},
                               "TCPDataPackageService": {"STATUS": "stop"}})
            conn.request("POST", "/changeStatus", body, {"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            if self.check_response(response):
                print('server stopped')
            else:
                print('server stopping failed')
            return 1
        except Exception as e:
            logger.error('there has been an exception in RestCLIClient stop_all ' + str(e))
            return -1

    def exit(self):
        self.killSwitch = True
        return 1

    def add_api_user(self):
        import uuid
        conn = http.client.HTTPConnection(connectionIP, connectionPort)
        username = input('please enter the username for the new user: ')
        token = input('please enter the token for the new user: ')
        if token is None:
            token = uuid.uuid4()
        else:
            pass
        conn.request("POST", "/APIUser", body = json.dumps({'username': username, "token": token}), headers={"Content-type": "application/json", "Accept": "text/plain"})
        response = conn.getresponse()
        print('user created\n'
              'username: ' + username + '\n'
              'token: ' + token)
        return response

    def remove_api_user(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            username = input('please enter username of user to be deleted: ')
            conn.request("DELETE", "/APIUser", body = json.dumps({'username': username}), headers={"Content-type": "application/json", "Accept": "text/plain"})
            response = conn.getresponse()
            print('user ' + username + ' has been deleted')
            return response
        except Exception as e:
            logger.error('there has been an exception in RestCLI Client remove_api_user ' + str(e))
            return -1

    def show_api_users(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("GET", "/APIUser")
            response = conn.getresponse()
            api_users = json.loads(response.read().decode("utf-8"))
            DPArray = []
            for dict in api_users['json_list']:
                values = list(dict.values())
                DPArray.append(values)
            table = tabulate.tabulate(DPArray, headers = ["Token", "Username"], tablefmt='psql')
            print(table)
            return 1

        except Exception as e:
            logger.error('there has been an exception in RestCLI Client show_api_users ' + str(e))
            return -1

    def show_DP(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("GET", "/DataPackageTable")
            response = conn.getresponse()
            DataPackages = json.loads(response.read().decode("utf-8"))
            DPArray = []
            for dict in DataPackages['json_list']:
                values = list(dict.values())
                DPArray.append(values)
            table = tabulate.tabulate(DPArray, headers = ["Keywords", "Name", "Index", "Privacy", "Size", "SubmissionDataTime", "SubmissionUser"], tablefmt='psql')
            print(table)
            return 1

        except Exception as e:
            logger.error("there has been an exception thrown in show DP function " + str(e))
            return -1

    def remove_DP(self):
        hash = input('Index of DataPackage to delete: ')
        conn = http.client.HTTPConnection(connectionIP, connectionPort)
        conn.request("DELETE", f"/DataPackageTable?Hash={hash}")
        response = conn.getresponse()
        DataPackages = json.loads(response.read().decode("utf-8"))
        if self.check_response(response):
            print('DP removed succesfully')
        else:
            print('DP removed failed')
        return 1

    def help(self):
        print('start_all: to begin all services type')
        print('start_CoT_service: to begin CoT service type')
        print('start_data_package_service: to begin data package service  type')
        print('stop_all: to terminate all services type')
        print('stop_CoT_service: to terminate CoT service type')
        print('stop_data_package_service: to begin data package service type')
        print("change_connection_info: change the address and port of the server you're connecting to")
        print('show_users: to show connected user information type')
        print('start_federation_client_service: to begin federation with another server')
        print('kill: to kill the full server type')
        print('kill: terminate all the services')
        print('show_DP: to show all DataPackages on the server')
        print('remove_DP: to remove a DataPackages on the server')
        print('add_api_user: create a user for the api')
        print('remove_api_user: delete an api user')
        print('show_api_users: show a table of all api users and their associated tokens')
        print('exit: exit the terminal')
        return 1

    def check_service_status(self):
        pass
        return 1

    def show_client_array(self):
        print(self.clientInformationArray)
        print('length is ' + len(self.clientInformationArray))
        return 1

    def show_users(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            conn.request("GET", "/Clients")
            data = conn.getresponse()
            data = data.read()
            conn.close()
            data = json.loads(data)
            data.insert(0, {'ip': 'IP', 'callsign': 'CALLSIGN', 'team': 'TEAM'})
            col_width = max(len(word) for row in data for word in row.values()) + 2  # padding
            for row in data:
                print("".join(word.ljust(col_width) for word in row.values()))
            return 1

        except Exception as e:
            print(e)

    def server_geochat(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            text = input('enter message: ')
            body = json.dumps({"detail": {'remarks': {"INTAG": text}}})
            conn.request("POST", "/SendGeoChat", body, {"Content-type": "application/json", "Accept": "application/json"})
            data = conn.getresponse()
            conn.close()
            if self.check_response(data):
                print('geochat sent')
            else:
                print('geochat failed to send')
        except Exception as e:
            pass

    def connection_message(self):
        try:
            conn = http.client.HTTPConnection(connectionIP, connectionPort)
            text = input('enter message: ')
            body = json.dumps({"detail": {'remarks': {"INTAG": text}}})
            conn.request("POST", "/ConnectionMessage", body, {"Content-type": "application/json", "Accept": "application/json"})
            data = conn.getresponse()
            conn.close()
            if self.check_response(data):
                print('geochat sent')
            else:
                print('geochat failed to send')
        except Exception as e:
            pass

    def verify_output(self, input, example=None):
        try:
            if example == None:
                if input == None or input == -1:
                    return False
                else:
                    return True

            else:
                if isinstance(input, example):
                    return True
                else:
                    return False
        except Exception as e:
            logger.error('there has been an exception in RestCLIClient verifying output ' + str(e))
            return False

    def kill(self):
        try:
            self.killSwitch = True
            return 1
        except Exception as e:
            logger.error('error in kill function '+str(e))

    def empty(self):
        return 1

    def receive_input(self):
        conn = http.client.HTTPConnection(connectionIP, connectionPort)
        while self.killSwitch == False:
            try:
                self.UserCommand = str(input('FTS$ ')) or 'empty'
                try:
                    function = getattr(self, self.UserCommand)
                except:
                    logger.error('this is not a valid command')
                functionOutput = function()
                if self.verify_output(functionOutput):
                    pass
                else:
                    raise Exception('function returned bad data')
            except Exception as e:
                logger.error('error in processing your request ' + str(e))
        self.stop_all()


if __name__ == "__main__":
    from FreeTAKServer.controllers.AsciiController import AsciiController
    AsciiController().ascii()
    RestCLIClient().receive_input()


#######################################################
# 
# dest.py
# Python implementation of the Class dest
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:48 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import Integer
from sqlalchemy import String


class Dest(Base):
# default constructor  def __init__(self):  

    # the call sign of the destination
    __tablename__ = "Dest"
    PrimaryKey = Column(Integer, primary_key=True, autoincrement=True)
    OwnerPrimaryKey = Column(String, ForeignKey("Marti.PrimaryKey"))
    callsign = Column(String)


#######################################################
# 
# SimpleCoT.py
# Python implementation of the Class SimpleCoT
# Generated by Enterprise Architect
# Created on:      17-Oct-2020 2:41:49 PM
# Original author: natha
# 
#######################################################
from FreeTAKServer.model.RestMessages.SimpleAPIMessageAbstract import SimpleAPIMessageAbstract

class SimpleCoT(SimpleAPIMessageAbstract):

# default constructor  def __init__(self):  

    pass


from FreeTAKServer.model.ExCheck.templateInstanceContentsData import templateInstanceContentsData
import datetime as dt

class templateInstanceContents:
    def __init__(self):
        self.data = templateInstanceContentsData()
        self.timestamp = ""
        self.creatorUid = ""
        self.settimestamp()
    def settimestamp(self, timestamp = None):
        DATETIME_FMT = "%Y-%m-%dT%H:%M:%S.%fZ"
        if timestamp == None:
            timer = dt.datetime
            now = timer.utcnow()
            zulu = now.strftime(DATETIME_FMT)
            self.timestamp = zulu
        else:
            self.timestamp = timestamp

    def gettimestamp(self):
        return self.timestamp

    def setcreatoruid(self, creatoruid):
        self.creatorUid = creatoruid

    def getcreatoruid(self):
        return self.creatorUid


#######################################################
# 
# RawConnectionInformation.py
# Python implementation of the Class RawConnectionInformation
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:47:28 AM
# Original author: Natha Paquette
# 
#######################################################


class RawConnectionInformation:
    def __init__(self):
        self.type = 'RawConnectionInformation'
        self.xmlString = ""
        self.ip = ""
        self.socket = ""



#######################################################
# 
# DataQueue.py
# Python implementation of the Class DataQueue
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:47:23 AM
# Original author: Natha Paquette
# 
#######################################################


class DataQueue:
# default constructor  def __init__(self):  

    __dataArray = "" 


#######################################################
# 
# MainSocket.py
# Python implementation of the Class MainSocket
# Generated by Enterprise Architect
# Created on:      21-May-2020 12:46:04 PM
# Original author: Natha Paquette
# 
#######################################################
import socket

class MainSocket:
    def __init__(self):  
        self.ip = "0.0.0.0" 
        self.port = "" 
        self.sock = "" 
        self.socketAF = socket.AF_INET
        self.socketSTREAM = socket.SOCK_STREAM 
        self.solSock = socket.SOL_SOCKET
        self.soReuseAddr = socket.SO_REUSEADDR
        self.sockProto = 1





from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class OrchestratorConstants:
    def __init__(self):
        self.CoTIP = "0.0.0.0"
        self.COTPORT = MainConfig.CoTServicePort
        self.APIPORTARG = '-APIPort'
        self.COTPORTARG = '-CoTPort'
        self.IPARG = "-IP"
        self.APIPORTDESC = 'the port address you would like FreeTAKServer to run receive datapackages on not'
        self.IPDESC = "the IP you would like FreeTAKServer to run receive connections on, must be set for private DataPackages to function"
        self.COTPORTDESC = 'the port you would like FreeTAKServer to run receive connections on'
        self.COTIPDESC = 'the IP you would like FreeTAKServer to run receive connections on'
        self.SSLCOTPORTDESC = 'the port you would like FreeTAKServer to run receive SSL connections on'
        self.SSLCOTIPDESC = 'the IP you would like FreeTAKServer to run receive SSL connections on'
        self.FULLDESC = 'FreeTAKServer startup settings'
        self.HEALTHCHECK = 'HealthCheck'
        self.LOCALHOST = '127.0.0.1'

        self.DEFAULTCONNECTIONGEOCHATOBJ = MainConfig.ConnectionMessage


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cotevent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import FreeTAKServer.model.protobufModel.detail_pb2 as detail__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cotevent.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0e\x63otevent.proto\x1a\x0c\x64\x65tail.proto\"\xed\x01\n\x08\x43otEvent\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63\x63\x65ss\x18\x02 \x01(\t\x12\x0b\n\x03qos\x18\x03 \x01(\t\x12\x0c\n\x04opex\x18\x04 \x01(\t\x12\x0b\n\x03uid\x18\x05 \x01(\t\x12\x10\n\x08sendTime\x18\x06 \x01(\x04\x12\x11\n\tstartTime\x18\x07 \x01(\x04\x12\x11\n\tstaleTime\x18\x08 \x01(\x04\x12\x0b\n\x03how\x18\t \x01(\t\x12\x0b\n\x03lat\x18\n \x01(\x01\x12\x0b\n\x03lon\x18\x0b \x01(\x01\x12\x0b\n\x03hae\x18\x0c \x01(\x01\x12\n\n\x02\x63\x65\x18\r \x01(\x01\x12\n\n\x02le\x18\x0e \x01(\x01\x12\x17\n\x06\x64\x65tail\x18\x0f \x01(\x0b\x32\x07.Detailb\x06proto3'
  ,
  dependencies=[detail__pb2.DESCRIPTOR,])




_COTEVENT = _descriptor.Descriptor(
  name='CotEvent',
  full_name='CotEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='CotEvent.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='access', full_name='CotEvent.access', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qos', full_name='CotEvent.qos', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='opex', full_name='CotEvent.opex', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='CotEvent.uid', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sendTime', full_name='CotEvent.sendTime', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='startTime', full_name='CotEvent.startTime', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='staleTime', full_name='CotEvent.staleTime', index=7,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='how', full_name='CotEvent.how', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lat', full_name='CotEvent.lat', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lon', full_name='CotEvent.lon', index=10,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hae', full_name='CotEvent.hae', index=11,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ce', full_name='CotEvent.ce', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='le', full_name='CotEvent.le', index=13,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='detail', full_name='CotEvent.detail', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=270,
)

_COTEVENT.fields_by_name['detail'].message_type = detail__pb2._DETAIL
DESCRIPTOR.message_types_by_name['CotEvent'] = _COTEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CotEvent = _reflection.GeneratedProtocolMessageType('CotEvent', (_message.Message,), {
  'DESCRIPTOR' : _COTEVENT,
  '__module__' : 'cotevent_pb2'
  # @@protoc_insertion_point(class_scope:CotEvent)
  })
_sym_db.RegisterMessage(CotEvent)


# @@protoc_insertion_point(module_scope)



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# point.py
# Python implementation of the CoT point
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):07 AM
# Original author: Corvo
# 
#######################################################
# Latitude referred to the WGS 84 ellipsoid in degrees
from FreeTAKServer.model.FTSModelVariables.PointVariables import PointVariables as vars

class Point(FTSProtocolObject):

    def __init__(self, le=vars().LE, ce=vars().CE, hae=vars().HAE, lon=vars().LON, lat=vars().LAT):
        self.le = le
        self.ce = ce
        self.hae = hae
        self.lon = lon
        self.lat = lat

    # ce getter 
    def getce(self): 
        return self.ce 

    # ce setter 
    def setce(self, ce):
        self.ce=ce 

    # le getter 
    def getle(self): 
        return self.le 

    # le setter 
    def setle(self,le):  
        self.le=le

    # lat getter 
    def getlat(self):
        return self.lat 

    # lat setter 
    def setlat(self, lat):  
        self.lat=lat

        # lon getter 
    def getlon(self):
        return self.lon 

    # lon setter 
    def setlon(self,lon):
        self.lon=lon
  
    def gethae(self):
        return self.hae

    def sethae(self,hae):
        self.hae = hae


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# Emergency.py
# Python implementation of the Class Emergency
# Generated by Enterprise Architect
# Created on:      13-Apr-2020 4:40:22 PM
# Original author: Corvo
# 
#######################################################
from FreeTAKServer.model.FTSModelVariables.EmergencyVariables import EmergencyVariables as vars

class Emergency(FTSProtocolObject):
    """An Emergency beacon the is continually send to all the connected clients until
    deactivated from the original creator
    """
    def __init__(self):  
        self.type = None
        self.alert = None
    # if true the Emergency beacon is canceled
        self.cancel = None
        self.INTAG = None

    @staticmethod
    def emergency_on(INTAG = vars.emergency_on().INTAG, TYPE = vars.emergency_on().TYPE, ALERT = vars.emergency_on().ALERT, CANCEL = vars.emergency_on().CANCEL):
        emergency = Emergency()
        emergency.settype(TYPE)
        emergency.setAlert(ALERT)
        emergency.setcancel(CANCEL)
        emergency.setINTAG(INTAG)
        return emergency

    @staticmethod
    def emergency_off(CANCEL = vars.emergency_off().CANCEL, INTAG = vars.emergency_off().INTAG):
        emergency = Emergency()
        emergency.setcancel(CANCEL)
        emergency.setINTAG(INTAG)
        return emergency

    def settype(self, type=None):
        self.type = type
    
    def gettype(self):
        return self.type

    def setAlert(self, alert=None):
        self.alert = alert

    def getAlert(self):
        return self.alert

    def setcancel(self, cancel=None):
        self.cancel = cancel

    def getcancel(self):
        return self.cancel

    def setINTAG(self, INTAG=None):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
#
# detail.py
# Python implementation of the Class detail
# Generated by Enterprise Architect
# Created on(FTSProtocolObject): 11-Feb-2020 11:08:07 AM
# Original author: Corvo
#
#######################################################
from FreeTAKServer.model.FTSModel.Takv import Takv
from FreeTAKServer.model.FTSModel.Uid import Uid
from FreeTAKServer.model.FTSModel.Precisionlocation import Precisionlocation
from FreeTAKServer.model.FTSModel._Group import _Group
from FreeTAKServer.model.FTSModel.Status import Status
from FreeTAKServer.model.FTSModel.Track import Track
from FreeTAKServer.model.FTSModel.Marti import Marti
from FreeTAKServer.model.FTSModel.Link import Link
from .Contact import Contact
from .Emergency import Emergency
from FreeTAKServer.model.FTSModel.Chat import Chat
from FreeTAKServer.model.FTSModel.Remarks import Remarks
from FreeTAKServer.model.FTSModel.Serverdestination import _Serverdestination as Serverdestination
from .Color import Color
from FreeTAKServer.model.FTSModel.Usericon import Usericon
from .Archive import Archive
from FreeTAKServer.model.FTSModel.Summary import Summary
from FreeTAKServer.model.FTSModel.Mission import Mission

class Detail(FTSProtocolObject):
    """An optional element used to hold CoT sub-schema. empty element
    """
    def __init__(self):
        pass

    @staticmethod
    def Connection():
        detail = Detail()
        detail.takv = Takv.connection()
        detail.contact = Contact.connection()
        detail.uid = Uid.connection()
        detail.precisionlocation = Precisionlocation.connection()
        detail._group = _Group.connection()
        detail.status = Status.connection()
        detail.track = Track.connection()
        detail.remarks = Remarks.connection()
        return detail

    @staticmethod
    def GeoChat():
        detail = Detail()
        detail._chat = Chat.geochat()
        detail.link = Link.geochat()
        detail.remarks = Remarks.geochat()
        detail._serverdestination = Serverdestination.geochat()
        detail.marti = Marti.drop_point()
        return detail

    @staticmethod
    def Ping():
        detail = Detail()
        detail.setuid(None)
        return detail

    @staticmethod
    def FederatedCoT():
        detail = Detail()
        detail.remarks = Remarks.FederatedCoT()
        detail.status = Status.FederatedCoT()
        return detail

    @staticmethod
    def Other():
        detail = Detail()
        detail.marti = Marti.other()
        return detail

    @staticmethod
    def emergencyOn():
        detail = Detail()
        detail.link = Link.emergency_on()
        detail.contact = Contact.emergency_on()
        detail.emergency = Emergency.emergency_on()
        return detail

    @staticmethod
    def emergencyOff():
        detail = Detail()
        detail.emergency = Emergency.emergency_off()
        return detail

    @staticmethod
    def dropPoint():
        detail = Detail()
        detail.archive = Archive.drop_point()
        detail.status = Status.drop_point()
        detail.usericon = Usericon.drop_point()
        detail.link = Link.drop_point()
        detail.color = Color.drop_point()
        detail.precisionlocation = Precisionlocation.drop_point()
        detail.contact = Contact.drop_point()
        detail.remarks = Remarks.drop_point()
        detail.marti = Marti.drop_point()
        detail.summary = Summary.drop_point()
        return detail

    @staticmethod
    def disconnect():
        detail = Detail()
        detail.link = Link.disconnect()
        return detail

    @staticmethod
    def UserUpdate():
        detail = Detail()
        detail._group = _Group.UserUpdate()
        detail.status = Status.UserUpdate()
        detail.takv = Takv.UserUpdate()
        detail.track = Track.UserUpdate()
        detail.contact = Contact.UserUpdate()
        detail.uid = Uid.UserUpdate()
        detail.precisionlocation = Precisionlocation.UserUpdate()
        return detail

    @staticmethod
    def SimpleCoT():
        detail = Detail()
        detail.contact = Contact.SimpleCoT()
        return detail

    @staticmethod
    def Presence():
        detail = Detail()
        detail.contact = Contact.Presence()
        detail._group = _Group.Presence()
        return detail

    @staticmethod
    def ExcheckUpdate():
        detail = Detail()
        detail.mission = Mission.ExcheckUpdate()
        return detail

    def setarchive(self, archive):
        self.archive = archive

    def getarchive(self):
        return self.archive

    def setsummary(self, summary):
        self.summary = summary

    def getsummary(self):
        return self.summary

    def setremarks(self, remarks):
        self.remarks = remarks

    def getremarks(self):
        return self.remarks

    def setlink(self, link):
        self.link = link

    def getlink(self):
        return self.link

    def setstatus(self, status):
        self.status = status

    def getstatus(self):
        return self.status

    def setusericon(self, usericon):
        self.usericon = usericon

    def getusericon(self):
        return self.usericon

    def setcolor(self, color):
        self.color = color

    def getcolor(self):
        return self.color

    def setcontact(self, contact):
        self.contact = contact

    def getcontact(self):
        return self.contact

    def sethierarchy(self, hierarchy):
        self.hierarchy = hierarchy

    def gethierarchy(self):
        return self.hierarchy

    def setuid(self, uid):
        self.uid = uid

    def getuid(self):
        return self.uid

    def settakv(self, takv):
        self.takv = takv

    def gettakv(self):
        return self.takv

    def set__group(self, group):
        self._group = group

    def get__group(self):
        return self._group

    def set_group(self, group):
        self._group = group

    def get_group(self):
        return self._group

    def setprecisionlocation(self, Precisionlocation):
        self.precisionlocation = Precisionlocation

    def getprecisionlocation(self):
        return self.precisionlocation

    def settrack(self, track):
        self.track = track

    def gettrack(self):
        return self.track

    def setmarti(self, marti):
        self.marti = marti

    def getmarti(self):
        return self.marti

    def setemergency(self, emergency):
        self.emergency = emergency

    def getemergency(self):
        return self.emergency

    def set_chat(self, chat):
        self._chat = chat

    def get_chat(self):
        return self._chat

    def set__chat(self, chat):
        self._chat = chat

    def get__chat(self):
        return self._chat

    def set_serverdestination(self, serverdestination):
        self._serverdestination = serverdestination

    def get_serverdestination(self):
        return self._serverdestination

    def set__serverdestination(self, serverdestination):
        self._serverdestination = serverdestination

    def get__serverdestination(self):
        return self._serverdestination






#######################################################
# 
# Contact.py
# Python implementation of the Class Contact
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:39 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy.orm import relationship

class Contact(Base):
# default constructor  def __init__(self):  

    __tablename__ = "Contact"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="contact")
    # The unit's voice call sign
    callsign = Column(String)
    # DSN number for this element (if applicable)
    dsn = Column(String)
    # e-mail address for this element (if applicable)
    email = Column(String)
    # TBD
    endpoint = Column(String)
    # The frequency (in MHz) on which the unit may be contacted via voice.
    freq = Column(String)
    # DNS-resolvable host name
    hostname = Column(String)
    iconsetpath = Column(String)
    # Amplifies the radio frequency information provided.  Contains the modulation
    # type for the communication.  (Coding tbd, should cover complex modulations such
    # as SINCGARS hopping, csma, etc...) am|fm
    modulation = Column(String)
    # Phone number for this element (if applicable)
    phone = Column(String)
    # Version tag for this sub schema.  Necessary to ensure upward compatibility with
    # future revisions.
    version = Column(String)




# !/usr/bin/python
import subprocess
try:
    from OpenSSL import crypto
except ImportError:
    subprocess.run(["pip3", "install", "pyopenssl"], capture_output=True)
from OpenSSL import crypto
import os
import getopt
import sys
import random
from shutil import copyfile
import uuid
from jinja2 import Template
import socket
import zipfile
import shutil
import pathlib
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

def generate_zip(server_address: str = None, server_filename: str = "pubserver.p12",
                     user_filename: str = "user.p12", cert_password: str = MainConfig.password) -> None:
    """
    A Function to generate a Client connection Data Package (DP) from a server and user p12 file in the current
    working directory.
    :param server_address: A string based ip address or FQDN that clients will use to connect to the server
    :param server_filename: The filename of the server p12 file default is pubserver.p12
    :param user_filename: The filename of the server p12 file default is user.p12
    :param cert_password: The password for the certificate files

    """
    pref_file_template = Template("""<?xml version='1.0' standalone='yes'?>
    <preferences>
        <preference version="1" name="cot_streams">
            <entry key="count" class="class java.lang.Integer">1</entry>
            <entry key="description0" class="class java.lang.String">FreeTAKServer_{{ server }}</entry>
            <entry key="enabled0" class="class java.lang.Boolean">false</entry>
            <entry key="connectString0" class="class java.lang.String">{{ server }}:8089:ssl</entry>
        </preference>
        <preference version="1" name="com.atakmap.app_preferences">
            <entry key="displayServerConnectionWidget" class="class java.lang.Boolean">true</entry>
            <entry key="caLocation" class="class java.lang.String">/storage/emulated/0/atak/cert/{{ server_filename }}</entry>
            <entry key="caPassword" class="class java.lang.String">{{ cert_password }}</entry>
            <entry key="clientPassword" class="class java.lang.String">{{ cert_password }}</entry>
            <entry key="certificateLocation" class="class java.lang.String">/storage/emulated/0/atak/cert/{{ user_filename }}</entry>
        </preference>
    </preferences>
    """)

    manifest_file_template = Template("""<MissionPackageManifest version="2">
       <Configuration>
          <Parameter name="uid" value="{{ uid }}"/>
          <Parameter name="name" value="FreeTAKServer_{{ server }}"/>
          <Parameter name="onReceiveDelete" value="true"/>
       </Configuration>
       <Contents>
          <Content ignore="false" zipEntry="{{ folder }}/fts.pref"/>
          <Content ignore="false" zipEntry="{{ folder }}/{{ server_filename }}"/>
          <Content ignore="false" zipEntry="{{ folder }}/{{ user_filename }}"/>	  
       </Contents>
    </MissionPackageManifest>
    """)

    manifest_file_parent_template = Template("""<MissionPackageManifest version="2">
           <Configuration>
              <Parameter name="uid" value="{{ uid }}"/>
              <Parameter name="name" value="FreeTAKServer_{{ server }}_DP"/>
           </Configuration>
           <Contents>
              <Content ignore="false" zipEntry="{{ folder }}/{{ internal_dp_name }}.zip"/>
           </Contents>
        </MissionPackageManifest>
        """)
    username = user_filename[:-4]
    random_id = uuid.uuid4()
    new_uid = uuid.uuid4()
    folder = "5c2bfcae3d98c9f4d262172df99ebac5"
    parentfolder = "80b828699e074a239066d454a76284eb"
    if MainConfig.UserConnectionIP == "0.0.0.0":
        hostname = socket.gethostname()
        server_address = socket.gethostbyname(hostname)
    else:
        server_address = MainConfig.UserConnectionIP
    pref = pref_file_template.render(server=server_address, server_filename=server_filename,
                                     user_filename=user_filename, cert_password=cert_password)
    man = manifest_file_template.render(uid=random_id, server=server_address, server_filename=server_filename,
                                        user_filename=user_filename, folder=folder)
    man_parent = manifest_file_parent_template.render(uid=new_uid, server=server_address,
                                                      folder=parentfolder,
                                                      internal_dp_name=f"{username.replace('./', '')}")
    if not os.path.exists("./" + folder):
        os.makedirs("./" + folder)
    if not os.path.exists("./MANIFEST"):
        os.makedirs("./MANIFEST")
    with open('./' + folder + '/fts.pref', 'w') as pref_file:
        pref_file.write(pref)
    with open('./MANIFEST/manifest.xml', 'w') as manifest_file:
        manifest_file.write(man)
    print("Generating Data Package: " + username + ".zip")
    copyfile(MainConfig.p12Dir, "./" + folder + "/" + server_filename)
    copyfile("./" + user_filename, "./" + folder + "/" + user_filename)
    zipf = zipfile.ZipFile(f"{username}.zip", 'w', zipfile.ZIP_DEFLATED)
    for root, dirs, files in os.walk('./' + folder):
        for file in files:
            zipf.write(os.path.join(root, file))
    for root, dirs, files in os.walk('./MANIFEST'):
        for file in files:
            zipf.write(os.path.join(root, file))
    zipf.close()
    shutil.rmtree("./MANIFEST")
    shutil.rmtree("./" + folder)
    # Create outer DP...because WinTAK
    if not os.path.exists("./" + parentfolder):
        os.makedirs("./" + parentfolder)
    if not os.path.exists("./MANIFEST"):
        os.makedirs("./MANIFEST")
    with open('./MANIFEST/manifest.xml', 'w') as manifest_parent:
        manifest_parent.write(man_parent)
    print(f"Generating Main Data Package: {username}_DP.zip")
    copyfile(f"./{username}.zip", f"./{parentfolder}/{username}.zip")
    zipp = zipfile.ZipFile(str(pathlib.PurePath(pathlib.Path(MainConfig.clientPackages), pathlib.Path(f"{username}.zip"))), 'w', zipfile.ZIP_DEFLATED)
    for root, dirs, files in os.walk('./' + parentfolder):
        for file in files:
            name = str(pathlib.PurePath(pathlib.Path(root), pathlib.Path(file)))
            zipp.write(name)
    for root, dirs, files in os.walk('./MANIFEST'):
        for file in files:
            zipp.write(os.path.join(root, file))
    zipp.close()
    shutil.rmtree("./MANIFEST")
    shutil.rmtree("./" + parentfolder)
    os.remove(f"./{username}.zip")


class AtakOfTheCerts:
    def __init__(self, pwd: str = MainConfig.password) -> None:
        """
        :param pwd: String based password used to secure the p12 files generated, defaults to atakatak
        """
        self.key = crypto.PKey()
        self.CERTPWD = pwd
        self.cakeypath = MainConfig.CAkey
        self.capempath = MainConfig.CA

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return None

    def generate_ca(self) -> None:
        """
        Generate a CA certificate
        """
        if not os.path.exists(self.cakeypath):
            print("Cannot find CA locally so generating one")
            ca_key = crypto.PKey()
            ca_key.generate_key(crypto.TYPE_RSA, 2048)
            cert = crypto.X509()
            cert.get_subject().CN = "CA"
            cert.set_serial_number(0)
            cert.set_version(2)
            cert.gmtime_adj_notBefore(0)
            cert.gmtime_adj_notAfter(31536000)
            cert.set_issuer(cert.get_subject())
            cert.add_extensions([crypto.X509Extension(b'basicConstraints', False, b'CA:TRUE'),
                                 crypto.X509Extension(b'keyUsage', False, b'keyCertSign, cRLSign')])
            cert.set_pubkey(ca_key)
            cert.sign(ca_key, "sha256")

            f = open(self.cakeypath, "wb")
            f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, ca_key))
            f.close()
            print("CA key Stored Here: " + self.cakeypath)

            f = open(self.capempath, "wb")
            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
            f.close()
            print("CA pem Stored Here: " + self.capempath)
        else:
            print("CA found locally, not generating a new one")

    def _generate_key(self, keypath: str) -> None:
        """
        Generate a new certificate key
        :param keypath: String based filepath to place new key, this should have a .key file extention
        """
        if os.path.exists(keypath):
            print("Certificate file exists, aborting.")
            print(keypath)
        else:
            print("Generating Key...")
            self.key.generate_key(crypto.TYPE_RSA, 2048)
            f = open(keypath, "wb")
            f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, self.key))
            f.close()
            print("Key Stored Here: " + keypath)

    def _generate_certificate(self, cn: str = "Server", pempath: str = MainConfig.pemDir, p12path: str = MainConfig.p12Dir) -> None:
        """
        Create a certificate and p12 file
        :param cn: Common Name for certificate
        :param pempath: String filepath for the pem file created
        :param p12path: String filepath for the p12 file created
        """
        if os.path.exists(pempath):
            print("Certificate File Exists, aborting.")
            print(pempath)
            return None
        cakey = crypto.load_privatekey(crypto.FILETYPE_PEM, open(self.cakeypath).read())
        capem = crypto.load_certificate(crypto.FILETYPE_PEM, open(self.capempath, 'rb').read())
        serialnumber = random.getrandbits(64)
        chain = (capem,)
        cert = crypto.X509()
        cert.get_subject().CN = cn
        cert.set_serial_number(serialnumber)
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(315360000)
        cert.set_issuer(capem.get_subject())
        cert.set_pubkey(self.key)
        cert.set_version(2)
        cert.sign(cakey, "sha256")
        print('cert genned')
        p12 = crypto.PKCS12()
        p12.set_privatekey(self.key)
        p12.set_certificate(cert)
        p12.set_ca_certificates(tuple(chain))
        p12data = p12.export(passphrase=bytes(self.CERTPWD, encoding='UTF-8'))
        with open(p12path, 'wb') as p12file:
            p12file.write(p12data)
            print("P12 Stored Here: " + p12path)

        if os.path.exists(pempath):
            print("Certificate File Exists, aborting.")
            print(pempath)
        else:
            f = open(pempath, "wb")
            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
            f.close()
            print("PEM Stored Here: " + pempath)

    def bake(self, cn: str, cert: str = "user") -> None:
        """
        Wrapper for creating certificate and all files needed
        :param cn: Common Name of the the certificate
        :param cert: Type of cert being created "user" or "server"
        """
        import os
        keypath = f"./{cn}.key"
        pempath = f"./{cn}.pem"
        p12path = f"./{cn}.p12"
        self._generate_key(keypath)
        self._generate_certificate(cn, pempath, p12path)
        os.remove(keypath)
        if cert.lower() == "server":
            copyfile(keypath, keypath + ".unencrypted")

    def bake_startup(self, cn: str = 'Server') -> None:
        self.generate_ca()
        self._generate_key(MainConfig.keyDir)
        print('key generated')
        self._generate_certificate(cn, MainConfig.pemDir, MainConfig.p12Dir)
        copyfile(MainConfig.keyDir, MainConfig.unencryptedKey)

    @staticmethod
    def copy_server_certs(server_name: str = "pubserver") -> None:
        """
        copy all the server files with of a given name to the FTS server cert location
        :param server_name: Name of the server/IP address that was used when generating the certificate
        """
        """python37_fts_path = MainConfig.MainPath
        python38_fts_path = MainConfig.MainPath
        if os.path.exists(python37_fts_path):
            dest = python37_fts_path
        elif os.path.exists(python38_fts_path):
            dest = python38_fts_path
        else:
            print("Cannot Find FreeTAKServer install location, cannot copy")
            return None
        if not os.path.exists(dest + "/Certs"):
            os.makedirs(dest + "/Certs")"""
        print("Copying ./" + server_name + ".key to :" + MainConfig.keyDir)
        copyfile("./" + server_name + ".key", MainConfig.keyDir)
        print("Done")
        print("Copying ./" + server_name + ".key to :" + MainConfig.unencryptedKey)
        copyfile("./" + server_name + ".key", MainConfig.unencryptedKey)
        print("Done")
        print("Copying ./" + server_name + ".pem to :" + MainConfig.pemDir)
        copyfile("./" + server_name + ".pem", MainConfig.pemDir)
        print("Done")
        """print("Copying ./ca.pem to :" + MainConfig.CA)
        copyfile("./ca.pem", MainConfig.CA)"""
        print("Done")

    def generate_auto_certs(self, ip: str, copy: bool = False) -> None:
        """
        Generate the basic files needed for a new install of FTS
        :param ip: A string based ip address or FQDN that clients will use to connect to the server
        :param copy: Whether to copy server files to FTS expected locations
        """
        self.bake("pubserver", "server")
        self.bake("user", "user")
        if copy is True:
            self.copy_server_certs()
        generate_zip(server_address=ip)

if __name__ == '__main__':
    """if True:
        AtakOfTheCerts().bake("testcert")
        generate_zip(user_filename='testcert.p12')"""
    VERSION = "0.3.5"
    help_txt = "This Python script is to be used to generate the certificate files needed for \n" \
               "FTS Version 1.3 and above to allow for SSL/TLS connections between Server and \n" \
               "Client.\n\n" \
               "This script works in the current working directory (the folder you are \n" \
               "currently in)\n\n" \
               "The .p12 files generated will need to be copied to ATAK clients\n" \
               "the default password set on the .p12 files is atakatak\n" \
               "The Server .key and .pem file will ne needed on the FTS server as per the MainConfig.py\n" \
               "The ca.pem is also needed for the MainConfig.py\n" \
               "the default password set on the .p12 files is atakatak, this can be overridden\n\n" \
               "Arguments:\n" \
               "-h --help : to open help\n" \
               "-v --version : to print the version number of the script\n" \
               "-p --password : to change the password for the p12 files from the default atakatak\n" \
               "-a --automated : to run the script in a headless mode to auto generate ca,server and user certs " \
               "for a fresh install\n" \
               "-c --copy : Use this in conjunction with -a to copy the server certs needed into the default location for FTS\n" \
               "-i --ip : The IP address of the server that clients will be accessing it on\n\n"
    AUTO = False
    COPY = False
    IP = False
    CERTPWD = "atakatak"
    cmd_args = sys.argv
    arg_list = cmd_args[1:]
    stort_opts = "avhci:p:"
    long_opts = ["automated", "version", "help", "copy", "ip", "password"]
    args, values = getopt.getopt(arg_list, stort_opts, long_opts)
    for current_arg, current_val in args:
        if current_arg in ("-h", "--help"):
            print(help_txt)
            exit(1)
        if current_arg in ("-v", "--version"):
            print(VERSION)
            exit(1)
        if current_arg in ("-p", "--password"):
            CERTPWD = current_val
        if current_arg in ("-a", "--automated"):
            AUTO = True
        if current_arg in ("-c", "--copy"):
            COPY = True
        if current_arg in ("-i", "--ip"):
            IP = current_val

    with AtakOfTheCerts() as aotc:
        aotc.generate_ca()
    if AUTO:
        if IP is False:
            IP = str(input("Enter IP address or FQDN that clients will use to connect to FTS: "))
        with AtakOfTheCerts() as aotc:
            aotc.generate_auto_certs(copy=COPY, ip=IP)
    else:
        server_p12 = None
        users_p12 = []
        server_question = input("Would you like to generate a server certificate? y/n ")
        if server_question.lower() == "y":
            with AtakOfTheCerts(CERTPWD) as aotc:
                IP = str(input("Enter IP address or FQDN that clients will use to connect to FTS: "))
                aotc.bake(cn=IP, cert="server")
                server_p12 = "./" + IP + ".p12"
            copy_question = input("Would you like to copy the server certificate files where needed for FTS? y/n ")
            if server_question.lower() == "y":
                aotc.copy_server_certs(server_name=IP)
        user_question = input("Would you like to generate a user certificate? y/n ")
        if user_question.lower() == "y":
            while True:
                with AtakOfTheCerts(CERTPWD) as aotc:
                    cn = input("Username: ")
                    if len(cn) == 0:
                        break
                    aotc.bake(cn, cert="user")
                    users_p12.append("./" + cn + ".p12")
                    cont = input("Generate another? y/n ")
                    if cont.lower() != "y":
                        break
            generate_zip_question = input(
                "Would you like to generate Data Packages for each user just created? y/n ")
            if generate_zip_question.lower() == "y":
                while server_p12 is None:
                    server_p12 = input("Enter path to server p12 file e.g ./pubserver.p12 : ")
                while IP is False:
                    IP = str(input("Enter IP address or FQDN that clients will use to connect to FTS: "))
                for user in users_p12:
                    generate_zip(server_address=IP, server_filename=server_p12, user_filename=user)